@@ -16,9 +16,10 @@ use File::Temp qw/ tempfile tempdir /;
 use FB3::Validator;
 use utf8;
 use Encode qw(encode_utf8 decode_utf8);
-use HTML::Entities;
+use XML::Entities;
+use XML::Entities::Data;
 
-our $VERSION = 0.03;
+our $VERSION = 0.04;
 
 =head1 NAME
 
@@ -44,8 +45,17 @@ my @BlockLevel =
 ('address','article','aside','blockquote','canvas','dd','div','dl','dt','fieldset','figcaption','figure','footer','form',
 'h1','h2','h3','h4','h5','h6',
 'header','hr','li','main','nav','noscript','ol','output','p','pre','section','table','tfoot','ul','video',
+#формально не блок-левел, но нам их тоже приводить к нормальному виду
+'th','tr','td'
 );
 
+my $AllEntities = XML::Entities::Data::all;
+delete $AllEntities->{'lt'};
+delete $AllEntities->{'gt'};
+delete $AllEntities->{'quot'};
+delete $AllEntities->{'apos'};
+delete $AllEntities->{'amp'};
+
 #Элементы, которые парсим в контенте и сохраняем в структуру 
 our $ElsMainList = {
   'span'=>undef,
@@ -55,26 +65,52 @@ our $ElsMainList = {
   'sup'=>undef,
   'code'=>undef,
   'img'=>undef,
-  'u'=>'underline', #переименование дочерней ноды
-  'underline'=>undef, #разрешение переименованной ноды
-  'b'=>'strong', #переименование дочерней ноды
-  'strong'=>undef, #разрешение переименованной ноды
+  'i'=>undef,
+  'u'=>undef,
+  'underline'=>undef,
+  'b'=>undef,
+  'strong'=>undef,
 };
 
+our $ElsMainList2={};
+map {$ElsMainList2->{$_}=undef;} keys %$ElsMainList;
+$ElsMainList2->{p}=undef;
+
 my %AllowElementsMain = (
+  'table' => {
+    'allow_attributes' => ['id'],
+    'allow_elements_inside' => {'tr'=>undef},
+  },
+  'tr' => {
+    'allow_attributes' => ['id','align'],
+    'allow_elements_inside' => {'th'=>undef,'td'=>undef},
+  },
+  'th' => {
+    'allow_attributes' => ['colspan','rowspan','align','valign'],
+    'allow_elements_inside' => $ElsMainList2,
+  },
+  'td' => {
+    'allow_attributes' => ['colspan','rowspan','align','valign'],
+    'allow_elements_inside' => $ElsMainList2,
+  },
+  'i' => {
+    'allow_attributes' => [],
+    'allow_elements_inside' => $ElsMainList,
+  },
   'strong' => {
     'allow_attributes' => ['id'],
     'allow_elements_inside' => $ElsMainList,
   },
   'underline' => {
-    'allow_attributes' => []  
+    'allow_attributes' => ['id'],
+    'allow_elements_inside' => $ElsMainList,
   },
   'em' => {
-    'allow_attributes' => [],
-        'allow_elements_inside' => {span=>undef}
+    'allow_attributes' => ['id'],
+    'allow_elements_inside' => $ElsMainList,
   },
   'u' => {
-    'allow_attributes' => []  
+    'allow_attributes' => ['id']  
   },
   'sup' => {
     'allow_attributes' => ['id']  
@@ -85,6 +121,7 @@ my %AllowElementsMain = (
   'p' => {
     'allow_attributes' => ['id'],
     'allow_elements_inside' => $ElsMainList,
+    'exclude_if_inside' => ['ul','ol','table'], #с этими вариантами лучше схлопнуться родительскому 'p'
   },
   'span' => {
     'allow_attributes' => ['id'],
@@ -125,6 +162,7 @@ my %AllowElementsMain = (
       'h4'=>undef,
       'h5'=>undef,
       'h6'=>undef,
+      'table'=>undef,
       'ul'=>undef, 
       'ol'=>undef, #полноправные элементы корня
       'p'=>undef,  # см. ^^^
@@ -614,7 +652,12 @@ sub FB3Create {
   foreach my $Section ($XC->findnodes( "/fb3-body/section/section", $Body), $XC->findnodes( "/fb3-body/section", $Body)) {
     $Section = $X->Transform2Valid(node=>$Section);
   }
-  
+
+  #финальное приведение table к валидному виду
+  foreach my $Table ($XC->findnodes( "/fb3-body//table", $Body)) {
+    $Table = $X->TransformTable2Valid(node=>$Table);
+  }
+
   open FHbody, ">$FNbody" or die "$FNbody: $!";
   print FHbody $Body->toString(1);
   close FHbody;
@@ -920,9 +963,6 @@ sub Content2Tree {
       load_ext_dtd => 0, # полный молчок про dtd
   );
 
-  $Content = HTML::Entities::decode_entities($Content);
-  $Content =~ s/\&/&#38;/g;
-
   my $NodeDoc = $XMLDoc->parse_string('<root_fb3_container>'.$Content.'</root_fb3_container>') || die "Can't parse! ".$!;
 
   my $RootEl = $NodeDoc->getDocumentElement;
@@ -1097,6 +1137,7 @@ sub RealPath {
   my $RealPath = undef;
   
   if ($RealPath = Cwd::realpath($Path)) {
+    $RealPath =~ s/%20/ /g;
     my $RealPath2 = $RealPath;
     $RealPath2 =~ s/#.*$//g;
     $RealPath = undef if !-f $RealPath2 && !-d $RealPath2;
@@ -1180,6 +1221,29 @@ sub quot {
   return $str;
 }
 
+sub qent {
+  my $X = shift;
+  my $Str = shift;
+
+  $Str =~ s/&#(0+)?60;/&lt;/g;
+  $Str =~ s/&#(0+)?62;/&gt;/g;
+  $Str =~ s/&#x(0+)?3e;/&gt;/gi;
+  $Str =~ s/&#x(0+)?3c;/&lt;/gi;
+
+  $Str =~ s/&#(0+)?38;/&amp;/g;
+  $Str =~ s/&#x(0+)?26;/&amp;/g;
+
+  $Str =~ s/&#(0+)?34;/&quot;/g;
+  $Str =~ s/&#x(0+)?22;/&quot;/g;
+
+  $Str =~ s/&#(0+)?39;/&apos;/g;
+  $Str =~ s/&#x(0+)?27;/&apos;/g;
+
+  XML::Entities::_decode_entities($Str, $AllEntities, 0);
+  $Str =~ s/&(?!amp;|quot;|apos;|lt;|gt;)/&amp;/gi;
+  return $Str;
+}
+
 sub html_trim {
   my $X = shift;
   my $str = shift;
@@ -1278,6 +1342,21 @@ sub Reaper {
   print "This method in package " . __PACKAGE__ . " and not defined in Processor class\n";
 }
 
+sub TransformTable2Valid {
+  my $X = shift;
+  my %Args = @_;
+  my $Node = $Args{'node'};
+
+  foreach my $TH ($Node->findnodes('./tr/th')) {
+    $X->Transform2Valid(node=>$TH);
+  }
+  foreach my $TD ($Node->findnodes('./tr/td')) {
+    $X->Transform2Valid(node=>$TD);
+  }
+
+  return $Node;
+}
+
 sub Transform2Valid {
   my $X = shift;
   my %Args = @_;
@@ -1291,7 +1370,7 @@ sub Transform2Valid {
   my $Wrap = XML::LibXML::Element->new("p");
 
   foreach my $Child ($Node->getChildnodes) {
-    if ($Child->nodeName =~ /^(p|ul|ol|title|subtitle|section)$/) {
+    if ($Child->nodeName =~ /^(p|table|ul|ol|title|subtitle|section)$/) {
       if ($Wrap->hasChildNodes) {
         #закроем текуший враппер и создадим новый
         $NewNode->addChild($Wrap->cloneNode(1));
@@ -1395,17 +1474,16 @@ sub CutLinkDiez {
 sub CorrectOuterLink{
   my $X = shift;
   my $Str = shift;
-  
+
   unless ($Str =~ /^(http|https|mailto|ftp)\:(.+)/i) {
     $X->Msg("Find not valid Link and delete [$Str]\n");
     return "";
   }
-  
-  
+
   my $Protocol = $1;
   my $Link = $2;
   $Link = $X->trim_soft($Link);
- 
+
   if ($Protocol eq 'mailto') {
     unless (ValidEMAIL($Link)) {
       $X->Msg("Find not valid Email and delete [".$Protocol.":".$Link."]\n");
@@ -1417,7 +1495,7 @@ sub CorrectOuterLink{
       return "";
     }
   }
-    
+
   return $Protocol.':'.$Link;
 }
 
@@ -1452,6 +1530,14 @@ sub ValidEMAIL{
   return $Email=~ /^[-+a-z0-9_]+(\.[-+a-z0-9_]+)*\@([-a-z0-9_]+\.)+[a-z]{2,10}$/i;
 }
 
+sub ShitFix {
+  my $X = shift;
+  my $Str = shift;
+  # /i вызывает невероятные тормоза к сожалению
+  $Str =~ s#<img(.*?)([^/])>((.|\r|\n)*?)</img>#<img$1$2 />$3#g; # <img>t</img> => <img/>t
+  return $Str;
+}
+
 sub ParseMetaFile {
   my $X = shift;
 
